/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <glib.h>
#include <aul.h>
#include <app_control_internal.h>

#include "rua.h"
#include "rua_private.h"
#include "rua_manager.h"
#include "rua_context_internal.h"
#include "rua_info_internal.h"

static int __convert_aul_error(int error)
{
	switch (error) {
	case AUL_R_EINVAL:
		return RUA_ERROR_INVALID_PARAMETER;
	case AUL_R_EILLACC:
		return RUA_ERROR_PERMISSION_DENIED;
	case AUL_R_ERROR:
		return RUA_ERROR_OUT_OF_MEMORY;
	default:
		return RUA_ERROR_IO_ERROR;
	}
}

API int rua_manager_foreach_rua_info(rua_manager_rua_info_cb callback,
		void *user_data)
{
	return rua_info_foreach(callback, user_data);
}

API int rua_manager_delete_rua_info(rua_info_h info)
{
	char *app_id;
	char *instance_id;
	int ret;

	if (!info) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	ret = rua_info_get_app_id(info, &app_id);
	if (ret != RUA_ERROR_NONE)
		return ret;

	ret = rua_info_get_instance_id(info, &instance_id);
	if (ret != RUA_ERROR_NONE) {
		free(app_id);
		return ret;
	}

	if (instance_id) {
		ret = rua_delete_history_with_instance_id(app_id,
				instance_id);
	} else {
		ret = rua_delete_history_with_pkgname(app_id);
	}

	free(instance_id);
	free(app_id);

	if (ret < 0) {
		_E("Failed to delete RUA history. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return RUA_ERROR_NONE;
}

API int rua_manager_delete_all_rua_info(void)
{
	int ret;

	ret = rua_clear_history();
	if (ret < 0) {
		_E("Failed to clear RUA history. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return RUA_ERROR_NONE;
}

API int rua_manager_get_app_control_from_rua_info(rua_info_h info,
		app_control_h *app_control)
{
	app_control_h handle;
	char *args;
	bundle *b;
	int ret;

	if (!info || !app_control) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	ret = rua_info_get_args(info, &args);
	if (ret != RUA_ERROR_NONE)
		return ret;

	b = bundle_decode((bundle_raw *)args, strlen(args));
	free(args);
	if (!b) {
		_E("Failed to decode bundle");
		return RUA_ERROR_OUT_OF_MEMORY;
	}

	ret = app_control_create_request(b, &handle);
	bundle_free(b);
	if (ret != APP_CONTROL_ERROR_NONE) {
		_E("Failed to create app_control handle");
		return RUA_ERROR_OUT_OF_MEMORY;
	}

	*app_control = handle;

	return RUA_ERROR_NONE;
}

API int rua_manager_get_rua_context_from_rua_info(rua_info_h info,
		rua_context_h *context)
{
	return rua_context_create_from_rua_info(info, context);
}

API int rua_manager_is_running(rua_context_h context, bool *running)
{
	return rua_context_is_running(context, running);
}

API int rua_manager_resume(rua_context_h context)
{
	return rua_context_resume(context);
}

API int rua_manager_terminate(rua_context_h context)
{
	return rua_context_terminate(context);
}
